@@ -186,11 +186,11 @@ main (int argc, char **argv)
   act.sa_handler = SIG_IGN;
   act.sa_mask    = empty_mask;
   act.sa_flags   = 0;
-  if (sigaction (SIGPIPE,  &act, 0) < 0)
+  if (sigaction (SIGPIPE,  &act, NULL) < 0)
     g_printerr ("Failed to register SIGPIPE handler: %s\n",
                 g_strerror (errno));
 #ifdef SIGXFSZ
-  if (sigaction (SIGXFSZ,  &act, 0) < 0)
+  if (sigaction (SIGXFSZ,  &act, NULL) < 0)
     g_printerr ("Failed to register SIGXFSZ handler: %s\n",
                 g_strerror (errno));
 #endif
@@ -378,7 +378,11 @@ main (int argc, char **argv)
 #else
   meta_verbose ("Compiled without startup notification\n");
 #endif
-
+#ifdef HAVE_COMPOSITE_EXTENSIONS
+  meta_verbose ("Compiled with composite extensions\n");
+#else
+  meta_verbose ("Compiled without composite extensions\n");
+#endif
   
   /* Load prefs */
   meta_prefs_init ();
@@ -421,18 +425,36 @@ main (int argc, char **argv)
   
   meta_ui_set_current_theme (meta_prefs_get_theme (), FALSE);
 
-  /* Try some panic stuff, this is lame but we really
-   * don't want users to lose their WM :-/
+  /* Try to find some theme that'll work if the theme preference
+   * doesn't exist.  First try Simple (the default theme) then just
+   * try anything in the themes directory.
    */
   if (!meta_ui_have_a_theme ())
     meta_ui_set_current_theme ("Simple", FALSE);
   
   if (!meta_ui_have_a_theme ())
-    meta_ui_set_current_theme ("Atlanta", FALSE);
-
-  if (!meta_ui_have_a_theme ())
-    meta_ui_set_current_theme ("Crux", FALSE);
-
+    {
+      const char *dir_entry = NULL;
+      GError *err = NULL;
+      GDir   *themes_dir = NULL;
+      
+      if (!(themes_dir = g_dir_open (METACITY_DATADIR"/themes", 0, &err)))
+        {
+          meta_fatal (_("Failed to scan themes directory: %s\n"), err->message);
+          g_error_free (err);
+        } 
+      else 
+        {
+          while (((dir_entry = g_dir_read_name (themes_dir)) != NULL) && 
+                 (!meta_ui_have_a_theme ()))
+            {
+              meta_ui_set_current_theme (dir_entry, FALSE);
+            }
+          
+          g_dir_close (themes_dir);
+        }
+    }
+  
   if (!meta_ui_have_a_theme ())
     meta_fatal (_("Could not find a theme! Be sure %s exists and contains the usual themes."),
                 METACITY_DATADIR"/themes");
